/**
 * Copyright (C) 2010 Cloudfarming <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package nl.cloudfarming.eventbus;

import java.util.EnumSet;

/**
 * Base {@link GuiEventProducer} implementation with access to the {@link GuiEventBus}
 * @author Timon Veenstra
 */
public abstract class BaseGuiEventProducer implements GuiEventProducer {

    protected EnumSet<? extends GuiEventKey> keys;
    protected EnumSet<? extends GuiEventKey> globalKeys;

    @SuppressWarnings("unchecked")
    protected BaseGuiEventProducer(Class<? extends Enum<? extends GuiEventKey>> keyClass) {
        this.keys = EnumSet.allOf((Class<? extends Enum>) keyClass);
        this.globalKeys = EnumSet.allOf((Class<? extends Enum>) GlobalGuiEventKey.class);
    }

    public void triggerEvent(final GuiEvent event){
        if (!keys.contains(event.getKey()) && !globalKeys.contains(event.getKey())) {
            throw new IllegalArgumentException("An event producer can only trigger events registered on that producer or global events");
        }
        GuiEventBus.getInstance().triggerEvent(event);
    }

    @Override
    public void triggerEvent(final GuiEventKey guiEventKey) {
        if (!keys.contains(guiEventKey) && !globalKeys.contains(guiEventKey)) {
            throw new IllegalArgumentException("An event producer can only trigger events registered on that producer or global events");
        }
        GuiEvent event = new GuiEvent() {

            @Override
            public GuiEventKey getKey() {
                return guiEventKey;
            }

            @Override
            public Object getContent() {
                throw new UnsupportedOperationException("This event contains no additional content.");
            }
        };

        GuiEventBus.getInstance().triggerEvent(event);
    }

    @Override
    public void triggerEvent(final GuiEventKey guiEventKey, final Object content) {
        if (!keys.contains(guiEventKey) && !globalKeys.contains(guiEventKey)) {
            throw new IllegalArgumentException("An event producer can only trigger events registered on that producer or global events");
        }
        GuiEvent event = new GuiEvent() {

            @Override
            public GuiEventKey getKey() {
                return guiEventKey;
            }

            @Override
            public Object getContent() {
                return content;
            }
        };

        GuiEventBus.getInstance().triggerEvent(event);
    }

    @Override
    public GuiEventKey[] getEventKeys() {
        return keys.toArray(new GuiEventKey[0]);
    }
}
