/**
 * Copyright (C) 2010 Cloudfarming <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.eventbus;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.openide.modules.ModuleInfo;
import org.openide.util.Lookup;
import org.openide.util.LookupEvent;
import org.openide.util.LookupListener;
import org.openide.util.lookup.AbstractLookup;
import org.openide.util.lookup.InstanceContent;
import org.openide.util.lookup.ProxyLookup;

/**
 * Central GuiEventBus.<p/>
 * Event mediator to be able to use eventing across netbeans modules.<p/>
 * All methods have default access modifier.
 * To use the GuiEventBus provide an implementation of {@link GuiEventProducer} 
 * (by extending {@link BaseGuiEventProducer}) and register the producer by putting
 * the full classname (incl package) of the implementation in the 
 * META-INF/services/nl.cloudfarming.eventbus.GuiEventProducer file<p/>
 * 
 * Listeners can be registered in META-INF/services/nl.cloudfarming.eventbus.GuiEventListener
 * or via the @ServiceProvider(service=GuiEventListener.class) annotation
 * 
 * 
 * @author Timon Veenstra
 */
public class GuiEventBus {

    // the instance
    private static GuiEventBus instance;
    //
    // internal listener registry which is combination of own register + listeners from serviceProviders
    //
    private static final InstanceContent lookupContent = new InstanceContent();
    private static final Lookup listenerLookup = new ProxyLookup(new AbstractLookup(lookupContent), Lookup.getDefault());
    private static Lookup.Result<GuiEventProducer> producers = Lookup.getDefault().lookupResult(GuiEventProducer.class);
    private final static Lookup.Result<ModuleInfo> modules = Lookup.getDefault().lookupResult(ModuleInfo.class);
    private static Map<String, ModuleInfo> moduleInfoMap;

    static {
        modules.addLookupListener(new LookupListener() {

            @Override
            public void resultChanged(LookupEvent event) {
                initializeModuleInformation();
            }
        });
    }

    private GuiEventBus() {
        initializeModuleInformation();
    }

    private static void initializeModuleInformation() {
        moduleInfoMap = new HashMap<String, ModuleInfo>();
        for (ModuleInfo info : modules.allInstances()) {
            moduleInfoMap.put(info.getDisplayName(), info);
        }
    }

    /**
     * 
     * @return instance of GuiEventBus
     */
    static GuiEventBus getInstance() {
        if (instance == null) {
            instance = new GuiEventBus();
        }
        return instance;
    }

    /**
     * Signal an event on the bus.
     * All listeners listening to the key in the event will be notified
     * 
     * @param event
     */
    void triggerEvent(GuiEvent event) {
        for (GuiEventListener listener : listenerLookup.lookupAll(GuiEventListener.class)) {
            if (moduleInfoMap.containsKey(listener.getModuleName())) {
                ModuleInfo info = moduleInfoMap.get(listener.getModuleName());
                if (info.isEnabled() && listener.listensTo(event.getKey())) {
                    listener.onEvent(event);
                }
            }

        }
    }

    List<String> getKeyList() {
        List<String> keyList = new ArrayList<String>();

        for (GuiEventProducer producer : producers.allInstances()) {
            for (GuiEventKey key : producer.getEventKeys()) {
                keyList.add(key.getKey());
            }
        }
        return keyList;
    }

    /**
     * Provides an alternative to the ServiceProvider registration method.
     *
     * @since 1.0.3
     * @param listener
     */
    public static void addListener(GuiEventListener listener) {
        lookupContent.add(listener);
    }
    
    /**
     * Provides an alternative to the ServiceProvider registration method.
     *
     * @since 1.0.3
     * @param listener
     */
    public static void removeListener(GuiEventListener listener) {
        lookupContent.remove(listener);
    }
}
